/**
 * IBM Confidential
 * OCO Source Materials
 * 5724-M24
 * Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
*/
package com.ibm.wbimonitor.sample.fabric.mediation;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.QueueConnectionFactory;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.rpc.handler.MessageContext;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.tools.ant.filters.StringInputStream;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.ibm.websphere.sib.SIMessage;
import com.ibm.websphere.sib.mediation.messagecontext.SIMessageContext;
import commonj.sdo.DataObject;

/**
 * Mediate CBE in Monitor Queue, send new message to Q for each array bo item
 * specified by XPath.
 * 
 * @author Yuan Zhao
 * @version 1.0
 */
public class SampleMediation implements
		com.ibm.websphere.sib.mediation.handler.MediationHandler {
    public static final String COPYRIGHT = "(C) Copyright IBM Corporation 2008.";  //$NON-NLS-1$
    
	private String loggerName = "MsgMediation"; //$NON-NLS-1$

	private String queueConnectionFactoryProperty = "QueueConnectionFactory"; //$NON-NLS-1$

	private String destinationProperty = "Destination"; //$NON-NLS-1$

	private String queueConnectionFactoryCtx;

	private String destinationCtx;

	private Logger log = Logger.getLogger(loggerName);

	public String getLoggerName() {
		return loggerName;
	}

	public void setLoggerName(String loggerName) {
		this.loggerName = loggerName;
	}

	public boolean handle(MessageContext msgCtx) {
		try {
			// get message from context
			SIMessageContext siMsgCtx = (SIMessageContext) msgCtx;
			SIMessage msg = siMsgCtx.getSIMessage();
			DataObject msgData = msg.getDataGraph().getRootObject();
			String cbe;

			// get queue connection factory name and destination name
			this.queueConnectionFactoryCtx = (String) msgCtx
					.getProperty(this.queueConnectionFactoryProperty); //$NON-NLS-1$
			this.destinationCtx = (String) msgCtx
					.getProperty(this.destinationProperty); //$NON-NLS-1$

			// validate context parameters
			if (this.queueConnectionFactoryCtx == null
					|| this.queueConnectionFactoryCtx == ""  //$NON-NLS-1$
					|| this.destinationCtx == null || this.destinationCtx == "") { //$NON-NLS-1$
				if (log.isLoggable(Level.SEVERE)) {
					log.severe(java.text.MessageFormat.format(Messages
							.getString("Mediation.ContextNotValid"),  //$NON-NLS-1$
							this.queueConnectionFactoryProperty
									+ this.destinationProperty));
				}
				return true;
			} 
//			else {
//				if (log.isLoggable(Level.INFO)) {
//					log.info(java.text.MessageFormat.format(Messages
//							.getString("Mediation.ContextProperties"),  //$NON-NLS-1$
//							this.queueConnectionFactoryProperty,
//							this.queueConnectionFactoryCtx,
//							this.destinationProperty, this.destinationCtx));
//				}
//			}
			
			// get cbe from SIMessage Data Object
			cbe = msgData.getString("data/value"); //$NON-NLS-1$
			// build message Document object
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			java.io.InputStream is = new StringInputStream(cbe);
			Document msgDoc = builder.parse(is);

			// mediation flag of message
			boolean mediationFlag;
			mediationFlag = isMsgMediated(msgDoc);
			// if message is already mediated, continue process message
			if (mediationFlag || msgDoc == null) {
				return true;
			}

			try {
				// create JMS context and connection/destination/session/message
				// producer
				InitialContext context = new InitialContext();
				QueueConnectionFactory cf = (QueueConnectionFactory) context
						.lookup(this.queueConnectionFactoryCtx); //$NON-NLS-1$
				Destination dest = (Destination) context
						.lookup(this.destinationCtx); //$NON-NLS-1$
				Connection conn = cf.createConnection();
				Session jmsSession = conn.createSession(false, 1);
				MessageProducer producer = jmsSession.createProducer(dest);

				// get message bpel id
				String msgBpelId = getMessageBpelId(msgDoc);

				if (msgBpelId == null || msgBpelId.equals("")) {
					return true;
				}

				// if message need mediation
				if (log.isLoggable(Level.INFO)) {
					log.info(Messages.getString("Mediation.MediationStart")); //$NON-NLS-1$
				}
				// set mediate message flag
				String newMsg = setMsgAsMediated(msgDoc);

				// mediate messges: for Receive or Reply message
				if (msgBpelId.equals(Constants.RECEIVE_MESSAGE_BPEL_ID)) {
					// get array bo item number
					Element paymentParent = SimpleDom.getElementBySimplePath(
							msgDoc, Constants.PAYMENT_BO_PARENT);
					NodeList paymentNodeList = paymentParent
							.getElementsByTagName(Constants.PAYMENT_ELEMENT);

					// payment number
					int paymentNumber = paymentNodeList.getLength();

					// send events according to bo number
					String currentMsg = newMsg;

					// store payments node into temp array
					ArrayList<Node> paymentNodeArray = new ArrayList<Node>();
					for (int k = 0; k < paymentNumber; k++) {
						Node node = paymentNodeList.item(k);
						paymentNodeArray.add(node);
					}

					// send message to queue
					if (paymentNumber > 1) {
						for (int i = 0; i < paymentNumber; i++) {
							TextMessage message = jmsSession
									.createTextMessage();
							message.setText(currentMsg);
							if (log.isLoggable(Level.INFO)) {
								log.info(Messages
										.getString("Mediation.SendingMessage")); //$NON-NLS-1$
							}
							producer.send(message);

							// remove the first payment bo
							paymentParent.removeChild(paymentNodeArray.get(i));
							currentMsg = transferDocumentToString(msgDoc);
						}
					} else {
						TextMessage message = jmsSession.createTextMessage();
						message.setText(currentMsg);
						if (log.isLoggable(Level.INFO)) {
							log.info(Messages
									.getString("Mediation.SendingMessage")); //$NON-NLS-1$
						}
						producer.send(message);
					}
				} else if (msgBpelId.equals(Constants.REPLY_MESSAGE_BPEL_ID)) {
					// get array bo item number
					Element paymentStatusParent = SimpleDom
							.getElementBySimplePath(msgDoc,
									Constants.PAYMENT_STATUS_BO_PARENT);
					// <wbi:applicationData>
					// <wbi:content wbi:name="TxInfAndSts"
					NodeList paymentStatusNodelist = paymentStatusParent
							.getElementsByTagName(Constants.WBI_CONTENT_ELEMENT);

					int childNumber = paymentStatusNodelist.getLength();

					// store payment status node to array and remove it from
					// document
					ArrayList<Node> nodeArray = new ArrayList<Node>();
					for (int k = childNumber - 1; k >= 0; k--) {
						Node node = paymentStatusNodelist.item(k);
						String value = getAttributeValue((Element) node,
								Constants.WBI_NAME_ATTRIBUTE);
						if ((value != null)
								&& value
										.equalsIgnoreCase(Constants.PAYMENT_STATUS_ELEMENT)) {
							nodeArray.add(node);
							paymentStatusParent.removeChild(node);
						}

					}

					// temp msg
					String currentMsg = newMsg;

					// recompose message and send message
					if (nodeArray.size() > 0) {
						for (int i = nodeArray.size() - 1; i >= 0; i--) {
							Node node = nodeArray.get(i);
							paymentStatusParent.appendChild(node);
							TextMessage message = jmsSession
									.createTextMessage();
							currentMsg = transferDocumentToString(msgDoc);
							message.setText(currentMsg);
							if (log.isLoggable(Level.INFO)) {
								log.info(Messages
										.getString("Mediation.SendingMessage")); //$NON-NLS-1$
							}
							producer.send(message);
							paymentStatusParent.removeChild(node);
						}
					} else {
						TextMessage message = jmsSession.createTextMessage();
						message.setText(currentMsg);
						if (log.isLoggable(Level.INFO)) {
							log.info(Messages
									.getString("Mediation.SendingMessage")); //$NON-NLS-1$
						}
						producer.send(message);
					}
				} else {
					TextMessage message = jmsSession.createTextMessage();
					message.setText(newMsg);
					if (log.isLoggable(Level.INFO)) {
						log
								.info(Messages
										.getString("Mediation.SendingMessage")); //$NON-NLS-1$
					}
					producer.send(message);
				}
			} catch (NamingException eNaming) {
				if (log.isLoggable(Level.INFO)) {
					log
							.info(Messages.getString("Mediation.ErrorHappened") + eNaming); //$NON-NLS-1$
				}
				//eNaming.printStackTrace();
			} catch (JMSException eJMS) {
				if (log.isLoggable(Level.INFO)) {
					log
							.info(Messages.getString("Mediation.ErrorHappened") + eJMS); //$NON-NLS-1$
				}
				//eJMS.printStackTrace();
			}
		} catch (Exception e) {
			if (log.isLoggable(Level.INFO)) {
				log.warning(Messages.getString("Mediation.ErrorHappened") + e);//$NON-NLS-1$
				//e.printStackTrace();
			}
		}
		return false;

	}

	// check wether message has already been mediated
	private boolean isMsgMediated(Document msgDoc) throws Exception {
		boolean isMediated = false;
		Element ele = msgDoc.getDocumentElement();
		String mediatedAtt = getAttributeValue(ele, Constants.MEDIATION_FLAG);
		if (mediatedAtt != null
				&& mediatedAtt.equals(Constants.MEDIATION_FLAG_VALUE))
			isMediated = true;
		return isMediated;
	}

	// get attribute value by attribute name, from Element object
	private String getAttributeValue(Element ele, String attributeName) {
		String value = null;
		if (ele.getAttributes().getNamedItem(attributeName) != null)
			value = ele.getAttributes().getNamedItem(attributeName)
					.getNodeValue();
		return value;
	}

	// set a mediated flag to msg root element attibute mediated="yes"
	private String setMsgAsMediated(Document msgDoc) throws Exception {
		String cbe = null;
		Element ele = msgDoc.getDocumentElement();
		ele.setAttribute(Constants.MEDIATION_FLAG,
				Constants.MEDIATION_FLAG_VALUE);
		cbe = transferDocumentToString(msgDoc);
		return cbe;
	}

	// transform a Document object to string
	private String transferDocumentToString(Document doc) throws Exception {
		String xmlString = null;

		DOMSource domSource = new DOMSource(doc);
		StringWriter writer = new StringWriter();
		StreamResult result = new StreamResult(writer);
		TransformerFactory tf = TransformerFactory.newInstance();
		Transformer transformer = tf.newTransformer();
		transformer.transform(domSource, result);
		xmlString = writer.toString();

		return xmlString;
	}

	// get Message bpel id
	private String getMessageBpelId(Document doc) throws Exception {
		String msgBpelId = null;
		Element ele = SimpleDom.getElementBySimplePath(doc,
				Constants.BPEL_ID_PATH);
		if (ele != null) {
			msgBpelId = ele.getTextContent().trim();
		}

		return msgBpelId;
	}
}
