/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 *
 * (C) Copyright IBM Corp. 2002, 2006 All Rights Reserved.
 * (C) Copyright Webify Solutions, Inc. 2002, 2006 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */
package com.ibm.websphere.fabric.banking.util;

import java.util.Map;
import java.util.logging.Logger;

import com.ibm.websphere.fabric.da.PendingRequest;
import com.ibm.websphere.fabric.da.context.Context;
import com.ibm.websphere.fabric.da.plugin.ContextExtractor;
import com.ibm.websphere.fabric.da.plugin.UnexpectedContentException;
import com.ibm.websphere.fabric.da.types.TypedValue;
import com.ibm.websphere.fabric.da.util.SdoXPathUtil;
import com.ibm.websphere.fabric.da.util.XPathSearchResult;
import commonj.sdo.DataObject;

/**
 * Sets the countrybased assertion for the PaymentNetworkSelection interface.
 * 
 * @author pmangesh
 * 
 */
public class PaymentNetworkSelectionContextExtractorImpl implements
		ContextExtractor {

	private static final String FABRIC_GOVERNANCE = "fc://fabric/governance/fabricproject-inst#71e99f5f-d80c-41a9-8956-365752f58d0d";

	private static final String FABRIC_GOVERNANCE_VALUE = "http://www.webifysolutions.com/2005/10/catalog/governance#FabricProject";

	private final Logger logger = Logger
			.getLogger(PaymentNetworkSelectionContextExtractorImpl.class
					.getName());

	/**
	 * Default constructor.
	 */
	public PaymentNetworkSelectionContextExtractorImpl() {
		super();
	}

	/**
	 * @see com.ibm.websphere.fabric.da.plugin.ContextExtractor#
	 *      extractContext(com.ibm.websphere.fabric.da.PendingRequest)
	 */
	public Context extractContext(PendingRequest pendingRequest)
			throws UnexpectedContentException {
		Context context = pendingRequest.getContext();

		DataObject requestDataObject = pendingRequest.getFirstBodyElement();

		XPathSearchResult result = SdoXPathUtil.getViaXpath(requestDataObject,
				"//CdtTrfTxInf[1]/PmtId/InstrId");

		String transactionId = (String) result.getValue();
		
		Map txnStateCountryMap = BpelContextHandler.getContextProperties();
		
		String country = ((StateCountry) txnStateCountryMap.get(transactionId))
				.getCountryCd();

		if (country != null) {
			TypedValue assertionValue = new TypedValue(country);

			context.setSelectionProperty(
					VendorPaymentUtil.PROP_HAS_COUNTRY_ASSERTION,
					assertionValue);
		}

		TypedValue governanceFabricProjectValue = new TypedValue(
				FABRIC_GOVERNANCE);

		context.setSelectionProperty(FABRIC_GOVERNANCE_VALUE,
				governanceFabricProjectValue);

		logger.warning("country came - " + country);
		return context;
	}
}