/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 *
 * (C) Copyright IBM Corp. 2002, 2006 All Rights Reserved.
 * (C) Copyright Webify Solutions, Inc. 2002, 2006 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */
package com.ibm.websphere.fabric.banking.util;

import com.ibm.websphere.fabric.da.util.SdoXPathUtil;
import com.ibm.websphere.fabric.da.util.XPathSearchResult;
import commonj.sdo.DataObject;

/**
 * VendorPaymentUtil has utility methods used in ContextExtrator classes.
 * 
 * @author pmangesh
 * 
 */
public final class VendorPaymentUtil {

    private VendorPaymentUtil() {

    }

    public static final String PROP_INTERFACE = "http://www.webifysolutions.com/context/interface";

    public static final String PROP_HAS_CITY_ASSERTION = "http://www.ibm.com/websphere/fabric/banking/payments/ifw/bom/assertions#hasSubRegion";

    public static final String PROP_HAS_COUNTRY_ASSERTION = "http://www.ibm.com/websphere/fabric/banking/payments/ifw/bom/assertions#hasCountryCd";
    
    public static final String PROP_TRANSACTION_AMOUNT_ASSERTION =	"http://www.ibm.com/websphere/fabric/banking/payments/iso20022/dict/5.1.2.1/assertions#hasTransactionAmount";

    /**
     * This method used for getting a Id for a StateCountry Bean.
     * 
     * @param isoMessage
     * @return
     */
    public static String getTransactionId(DataObject isoMessage) {

        XPathSearchResult result = SdoXPathUtil.getViaXpath(isoMessage,
                "//PmtInf[1]/CdtTrfTxInf[1]/PmtId/InstrId");
        return (String) result.getValue();
    }

    /**
     * This method populates the StateCountry bean from a iso request.
     * 
     * @param isoMessage
     * @return
     */
    public static StateCountry getStateCountryBean(DataObject isoMessage) {

        XPathSearchResult stateCdResult = SdoXPathUtil.getViaXpath(isoMessage,
                "//PmtInf[1]/CdtTrfTxInf[1]/Cdtr/PstlAdr/PstCd");
        String stateCd = ((String) stateCdResult.getValue()).substring(0, 2);

        XPathSearchResult countryCdResult = SdoXPathUtil.getViaXpath(isoMessage,
                "//PmtInf[1]/CdtTrfTxInf[1]/Cdtr/PstlAdr/Ctry");
        String countryCd = (String) countryCdResult.getValue();

        // Fix Required: For the states other than
        // TX and TN , value has been hardcoded to some dummy value.
        if (!(stateCd.equals("TX") || stateCd.equals("TN"))) {
            stateCd = "ZZ";
        }

        // Fix Required: For the country other than
        // US and GB, value has been hardcoded to some dummy value.
        if (!(countryCd.equals("US") || countryCd.equals("UK"))) {
            countryCd = "IN";
        }

        return new StateCountry(stateCd, countryCd);
    }
}