
package com.ibm.websphere.fabric.banking.mortgageRefinancing.mock.service;

import commonj.sdo.DataObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;



public class ServiceImplementations implements ServiceInterfaces {

	private static Log _log = LogFactory.getLog(ServiceImplementations.class);

	
	public DataObject ApplyRiskScale(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface: ArrangementServicing
		 * Operation: applyRiskRatingScale
		 * Input Business Object: ApplyRiskRatingScaleRequest
		 * Output Business Object: ApplyRiskRatingScaleResponse
		 * 
		 * DataObject DO1 should be of type ApplyRiskRatingScaleRequest
		 * 
		 */
		
		debug("Applying Risk Scale for Request Id: " +DO1.getDataObject("requestHeader").getString("id")) ;
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/ArrangementServicing", "ApplyRiskRatingScaleResponse");
	}

	public DataObject CheckCreditScore(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface: Customer
		 * Operation: getCreditRating
		 * Input Business Object:  GetCreditRatingRequest
		 * Output Business Object: GetCreditRatingResponse
		 * 
		 * DataObject DO1 should be of type GetCreditRatingRequest
		 * 
		 */
		debug("Credit Rating Processed : " + DO1);
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/Customer", "GetCreditRatingResponse");
	}

	public DataObject CheckOFAC(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface: RetrieveDesignatedPersonsListPT
		 * Operation: retrieveDesginatedPersonsList
		 * Input Business Object: RetrieveDesignatedPersonsListRequest
		 * Output Business Object: RetrieveDesignatedPersonsListResponse
		 * 
		 * DataObject DO1 Should be of type RetrieveDesignatedPersonsListRequest
		 */
		
		debug("OFAC Check Processed for Request Id: " +DO1.getDataObject("requestHeader").getString("id"));
		
		// Set Response
		
		// Create DataObjects
		 DataObject retrieveDesignatedPersonsListResponse = IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/RestrictedClassificationManagement", "RetrieveDesignatedPersonsListResponse");
		 DataObject responseHeader = IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/base","ResponseHeader");
		
		 // Populate ResponseHeader
		 responseHeader.setString("id", DO1.getDataObject("requestHeader").getString("id"));
		 responseHeader.setString("refidRequest",DO1.getDataObject("requestHeader").getString("id"));
		 
		 // Populate RetrieveDesignatedPersonsListResponse
		 retrieveDesignatedPersonsListResponse.setDataObject("responseHeader", responseHeader);
		 retrieveDesignatedPersonsListResponse.setDataObject("designatedPersonsList", null); 
		
		 debug("The Customer is not in OFAC List");
		
		return retrieveDesignatedPersonsListResponse;
	}

	public DataObject DetermineArragementPlan(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface: ArrangementSetup
		 * Operation: offerProductArrangement
		 * Input Business Object: OfferProductArrangementRequest
		 * Output Business Object: OfferProductArrangementResponse
		 * 
		 * DataObject DO1 should be of type OfferProductArrangementRequest
		 * 
		 */
		
		debug("Arrangement Plan is determined for Request Id: " +DO1.getDataObject("requestHeader").getString("id"));
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/ArrangementSetup", "OfferProductArrangementResponse");
	}

	public DataObject ForecastArrangementRisk(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface: SpecificRiskManagement
		 * Operation: forecastArrangementRisk
		 * Input Business Object: ForecastArrangementRiskRequest
		 * Output Business Object: ForecastArrangementRiskResponse
		 * 
		 * DataObject DO1 should be of type ForecastArrangementRiskRequest
		 * 
		 */
		
		debug("Forecasting Arrangement Risk for Request Id: " +DO1.getDataObject("requestHeader").getString("id"));
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/SpecificRiskManagement", "ForecastArrangementRiskResponse");
	}

	
	public DataObject NotifyCustomerofApproval(DataObject DO1) {
		/* 
		 * SCA Details
		 * Interface:  		Communication
		 * Operation:  		sendCommunication
		 * BusinessObject:  SendCommunicationRequest 	
		 *
		 * DO1 Should be of type SendCommunicationRequest
		 */
		
		debug("Customer is Notified : " +DO1.getString("details")
		        + " for request: "  + DO1.getDataObject("requestHeader").getString("id") 
		        + " on Date: " +DO1.getDate("date"));
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/CommunicationDispatch", "SendCommunicationResponse");
	}

	public DataObject NotifyMortgageRefinancingIdentificationToCustomer(
			DataObject DO1) {
		 
		/* 
		 * SCA Details
		 * Interface:  		Communication
		 * Operation:  		sendCommunication
		 * BusinessObject:  SendCommunicationRequest 	
		 *
		 * DO1 Should be of type SendCommunicationRequest
		 */
				
		debug("Customer is Notified of Mortgage Refinancing Id: " +DO1);
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/CommunicationDispatch", "SendCommunicationResponse");
	}

	public DataObject NotityCustomerofRejection(DataObject DO1) {
		/* 
		 * SCA Details
		 * Interface:  		Communication
		 * Operation:  		sendCommunication
		 * BusinessObject:  SendCommunicationRequest 	
		 *
		 * DO1 Should be of type SendCommunicationRequest
		 */
		
		debug("Customer is Notified : " +DO1.getString("details")
		        + " for request: "  + DO1.getDataObject("requestHeader").getString("id") 
		        + " on Date: " +DO1.getDate("date"));
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/CommunicationDispatch", "SendCommunicationResponse");
	}

	public DataObject ProposeMortgageRefinancingOption(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface: RecordArrangementConditionsPT
		 * Operation: recordArrangementConditions
		 * Input Business Object: RecordArrangementConditionsRequest
		 * Output Business Object: RecordArrangementConditionsResponse
		 * 
		 * DataObject DO1 Should be of type RecordArrangementConditionsRequest
		 * 
		 */
		
		debug("Proposing Mortgage Refinancing Option for Request Id: " + DO1.getDataObject("requestHeader").getString("id"));
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/ArrangementSetup", "RecordArrangementConditionsResponse");
	}

	public DataObject SelectProductCondition(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface: RecordArrangementConditionsPT
		 * Operation: recordArrangementConditions
		 * Input Business Object: RecordArrangementConditionsRequest
		 * Output Business Object: RecordArrangementConditionsResponse
		 * 
		 * DataObject DO1 Should be of type RecordArrangementConditionsRequest
		 * 
		 */
		debug("Product Condition Selected for request Id: " + DO1.getDataObject("requestHeader").getString("id"));
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/ArrangementSetup", "RecordArrangementConditionsResponse");
	}

	public DataObject UpdateStatustoApproved(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface:  Arrangement
		 * Operation:  acceptArrangement
		 * Input BusinessObject : AcceptArrangementRequest
		 * Output BusinessObject: AcceptArrangementResponse
		 * 
		 * DO1 Should be of type AcceptArrangementRequest
		 * 
		 */
		debug("Customer is Notified of Mortgage Refinancing Approval: " +DO1.getString("details")
		        + " for request: "  + DO1.getDataObject("requestHeader").getString("id") 
		        + " on Date: " +DO1.getDate("date"));
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/Arrangement", "AcceptArrangementResponse");
		
	}

	public DataObject UpdateStatustoRejected(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface:  Arrangement
		 * Operation:  cancelArrangement
		 * Input BusinessObject : CancelArrangementRequest
		 * Output BusinessObject: CancelArrangementResponse
		 * 
		 * DO1 Should be of type CancelArrangementRequest
		 * 
		 */
		debug("Customer is Notified of Mortgage Refinancing Rejection: " +DO1.getString("details")
		        + " for request: "  + DO1.getDataObject("requestHeader").getString("id") 
		        + " on Date: " +DO1.getDate("date"));
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/Arrangement", "CancelArrangementResponse");
	}

	public DataObject VerifyAssetAndLiabilityDetails(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface:  CalculateAssetValuePT
		 * Operation:  calculateAssetValue
		 * Input BusinessObject : CalculateAssetValueRequest
		 * Output BusinessObject: CalculateAssetValueResponse
		 * 
		 * DO1 Should be of type CalculateAssetValueRequest
		 * 
		 */
		
		debug("Verifying Asset And Liability Details "
		        + " for request: "  + DO1.getDataObject("requestHeader").getString("id")); 
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/ProductArrangementServicing", "CalculateAssetValueResponse");
	}

	public DataObject VerifyMortgageAccountInformation(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface:  ArrangementServicing
		 * Operation:  verifyArrangementDetails
		 * Input BusinessObject : VerifyArrangementDetailsRequest
		 * Output BusinessObject: VerifyArrangementDetailsResponse
		 * 
		 * DO1 Should be of type VerifyArrangementDetailsRequest
		 * 
		 */
		
		 debug("Mortgage Account Information Verified for Request Id: " + DO1.getDataObject("requestHeader").getString("id"));
		 
		 // Set Response
		 
		 // Create DataObjects
		 DataObject verifyArrangementDetailsResponse = IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/ArrangementServicing", "VerifyArrangementDetailsResponse");
		 DataObject responseHeader = IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/base","ResponseHeader");
		
		 // Populate ResponseHeader
		 responseHeader.setString("id", DO1.getDataObject("requestHeader").getString("id"));
		 responseHeader.setString("refidRequest",DO1.getDataObject("requestHeader").getString("id"));
		 
		 // Populate VerifyArrangementDetailsResponse
		 verifyArrangementDetailsResponse.setDataObject("responseHeader", responseHeader);
		 verifyArrangementDetailsResponse.setBoolean("arrangementValid", true); 
		 debug("Mortage Account Information is Valid");
		 
		return verifyArrangementDetailsResponse;
	}

	public DataObject VerifyPropertyDetails(DataObject DO1) {
		/*
		 * SCA Details
		 * Interface: GetMarketValuePT
		 * Operation: getMarketValue
		 * Input Business Object: GetMarketValueRequest
		 * Output Business Object: GetMarketValueResponse
		 * 
		 * DataObject DO1 should be of type GetMarketValueRequest
		 * 
		 */
		
		debug("Property Details Verified for Request Id: " + DO1.getDataObject("requestHeader").getString("id"));
		
		return IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2010/07/banking/ifw/81/schema/msg/RealProperty", "GetMarketValueResponse");
	}

	private void debug(String s) {
		System.out.println(s);
		_log.info(s);
	}
	

}
